/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: parse.c                                        *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         These routines will parse the command lines and put  *
*         values into fields in the udp_data structure.        *
*                                                              *
*       Input:                                                 *
*         The command lines received by the lp_filter main     *
*         program and the rteld main program provide the input *
*         necessary for this routine.                          *
*                                                              *
*       Output:                                                *
*         The udp_data structure server and service name       *
*         fields are filled with the information needed to     *
*         process this job.                                    *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  20-may-92  Issue an error message to the user for
                   invalid command line arguments.

   RKL  11-apr-92  Added parsing -i (immediate) option to force
                   the rteld daemon process to open the network
                   connection to the server immediately.

   RKL  06-jan-92  Added ability to parse telnet option on
                   command line.

   RKL  09-jan-91  Changed RTAP to RTEL.

   KCS  07-may-91  Merged aix_parse to here (no need to carry extra files)
    
   KCS  07-may-91  Added message field to errs() call, and err_handler.

   KCS  05-jun-91  Merged in routine to parse rteld command line.

 */

#include <udpinc.h>

int lp_parse(argc, argv, users)
int argc;
char *argv[];
udp_data *users;
{
    char *opts=argv[5], *srch = opts;
    int rval;
    
    users->host[0] = 0;
    
    strcpy (users->usernam, argv[2]);
    users->copies = atoi(argv[4]);
    
    if (argc > 6) {  /* then a filename is provided */
	strcpy(users->fname,argv[6]);
    }
    else {
	errs(users, RTEL_NOFILE, "parse:nofilnam");
	return PROC_ERR;
    }
    
    /*  Scan through the list of options to find something of the
	form service@server.  If this option string is found,
	extract the server and service name and put them into
	the udp_data structure.  */
	
    while (*srch){
	if (*srch == '@'){
	    *srch++ = 0;
	    strcpy(users->svcnm, opts);
	    strcpy(users->host, srch);
	}
	srch++;
    }
    return OK;
}



int aix_parse(argc, argv, users)
int argc;
char *argv[];
udp_data *users;
{
    strcpy(users->fname, argv[1]);
    users->host[0] = 0;
    
    return OK;
}




int rteld_parse(argc, argv, users)
int argc;
char *argv[];
udp_data *users;
{
    char *ptr, saved_char, *tmpptr;
    int i, process = 0;
    
    /* We must have at least one argument. */

    if (argc <= 1){
	return(HELP);
    }

    /* Now step through a loop and process each argument.  They do not
       have to be in any particular order, so take them as they come. 

       Do a bit of checking.  If the first character is not a "-" 
       send back an error message saying no good.  */

    for (i=1; i < argc; i++) {
	ptr = argv[i];
	if((*ptr != '-') || (strlen(ptr) < 2)){
	    printf("Unknown or invalid command line option %s\r\n", ptr);
	    return(HELP);
	}
	
	ptr++; /* skip over the "-" */
	switch (*ptr){

	    case 'b':{  /* become a system daemon */
		users->pty_flags |= BE_DAEMON;
		break;
	    }
	    
	    case 'c':{  /* parse -cservice[:port]@host*/
		ptr++;
		tmpptr = ptr;
		while (*ptr && (*ptr != '@') && (*ptr != ':'))ptr++;
		if(*ptr){
		    saved_char = *ptr;
		    *ptr = 0;
		    strcpy(users->svcnm, tmpptr);
		}

		ptr++;
		tmpptr = ptr;
		if(saved_char == ':'){
		    while(*ptr && (*ptr != '@'))ptr++;
		    if(*ptr){
			*ptr = 0;
			strcpy(users->hport, tmpptr);
			ptr++;
		    }
		}

		strcpy(users->host, ptr);
		break;
	    }
	    
	    case 'd':{  /* pseudo-tty device name */
		ptr++;
		strcpy(users->ttydevs, ptr);
		break;
	    }
	    
	    case 'f':{  /* remove stale kill files */
		if(process){
		    printf("Only one function can be specified\r\n");
		    return(HELP);
		}
		process = CLEAN;
		break;
	    }

	    case 'h':{  /* help */
		process = HELP;
		break;
	    }
	    
	    case 'i':{  /* immediate connection */
		users->pty_flags = CONN_IMED;
		break;
	    }
		
	    case 'k':{  /* kill a daemon process */
		if(process){
		    printf("Only one function can be specified\r\n");
		    return(HELP);
		}
		ptr++;
		strcpy(users->ttydevm, ptr);
		process = KILL;
		break;
	    }
	    
	    case 'p':{  /* password for connection */
		ptr++;
		strcpy(users->passwd, ptr);
		break;
	    }

	    case 'q':{  /* allow queueing */
		users->pty_flags = QUEUE_PTY;
		break;
	    }
	    
	    case 'r':{  /* show currently running rteld daemons */
		if(process){
		    printf("Only one function can be specified\r\n");
		    return(HELP);
		}
		process = REVIEW;
		break;
	    }
	    
	    case 's':{  /* run setup stuff */
		if(process){
		    printf("Only one function can be specified\r\n");
		    return(HELP);
		}
		process = SETUP;
		break;
	    }
	    
	    case 't':{  /* use telnet socket */
		users->tcp_socket = SOCK_RTELIAC;
		break;
	    }

	    case 'v':{  /* show rtel version */
		process = VERS;
		break;
	    }
	    
	    default:{
		printf("Unknown command line option %s\r\n", ptr);
		process = HELP;
		break;
	    }
	}
    }
    
    if (!process){
	return TALK;  /* default */
    }
    return process;
}
