/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: process_file                                   *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will actually move file data to the     *
*         server.                                              *
*                                                              *
*       Input:                                                 *
*         The fd field in the udp_data structure contains the  *
*         file descriptor to use to read from the input        *
*         stream.                                              *
*                                                              *
*       Output:                                                *
*         The data received on the input fd will be sent to    *
*         the server.                                          *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  07-jan-92  Cleaned up and reorganized data buffer
                   conversion.

   RKL  29-oct-91  Added ifdef for SVR4 type systems around problem
                   case statement.

   KCS  29-oct-91  ditched the pause_abit() call and replaced it with a
                   call to select, providing only the write_fd field, which
		   says to hang until the socket is available to be written
		   on...

   KCS  07-may-91  Added message field to errs() call, and err_handler.

   RKL  02-apr-91  Fix bogus problem with binary type I/O.  Use
                   array of bytes for fread rather than array
		   of blocks.

   RKL  28-feb-91  Use BUFFSIZE constant for file I/O buffer size.

   RKL  10-jan-91  Cleaned up network write return status processing.

   RKL  09-jan-91  Changed RTAP to RTEL.

 */

#include <udpinc.h>

int process_file(users)
udp_data *users;
{
    extern int errno;

    fd_set efd, rfd, wfd;
    char *buf1, tab_buf[BUFFSIZE*8];
    int index, rval, res, readnew, stat, tabptr;
  
    /* The input file has been opened by main and the fildesc is in 
       users->fdp.  Also, by this point we should be connected to the
       service, so all we need now is to read and send the file until
       EOF is encountered.  To do this, spin forever in select loop
       and when EOF is reached do a break.

       Note that even though we are doing a select on write, it is
       possible that the write could block.  Set the NDELAY option
       on the network socket just in case.  */

    res = fcntl(users->tcp_fd, F_SETFL, O_NDELAY);
    if (res < OK){
	errs(users, errno, "process_file:delaynset");
	return(PROC_ERR);
    }
    
    readnew = TRUE;
    
    while (TRUE) {
	FD_ZERO(&efd);
	FD_ZERO(&rfd);
	FD_ZERO(&wfd);
	FD_SET(users->tcp_fd, &efd);
	FD_SET(users->tcp_fd, &rfd);
	FD_SET(users->tcp_fd, &wfd);
	
	res = select(getdtablesize(), &rfd, &wfd, &efd, 0);
	
	if (res < OK) {  /* got interrupt */
	    if (errno == EINTR) continue;
	    errs(users, errno, "process_file:badselect");
	    return(PROC_ERR);
	}
	
	if(FD_ISSET(users->tcp_fd, &efd)) {
	    errs(users, RTEL_FINNED, "process_file:excptbad");
	    return(PROC_ERR);
	}
	
	if(FD_ISSET(users->tcp_fd, &rfd)) {
	    if ((stat = read(users->tcp_fd, tab_buf, BUFFSIZE)) <= 0) {
		if (errno == EINTR) continue;
		errs(users, RTEL_FINNED, "process_file:readbad");
		errs(users, errno, "process_file:readbad");
		return(PROC_ERR);
	    }
	}
	
	if (FD_ISSET(users->tcp_fd, &wfd)) {
	    if(readnew){
		buf1 = (char *)users->usr_buf;
		if(users->flags & OPT_BINARY){
		    if ((rval = fread(buf1, 1, BUFFSIZE, users->fdp)) == 0) break;
		}
		else {
		    if (fgets(buf1, BUFFSIZE, users->fdp) == NULL) break;
		    rval = strlen(buf1);
		}

		/* Now massage the buffer:

			   Tab (0x09) --------------> <VT>
                                                      <SP> ... <SP>    (expandtabs specified)

		           Line feed (0x0a) --------> <LF>             (binary)
			                              <CR> <LF>        (no telnet padding)
			                              <CR> <NULL> <LF> (telnet padding)

			   Carriage return (0x0d) --> <CR>             (binary)
                                                      <CR> <NULL>      (telnet padding)

			   IAC (0xff) --------------> <FF>
			                              <FF> <FF>        (telnet padding)

                   All other characters are passed through with no massaging.  */

		tabptr = 0;
		for(index=0; index < rval; index++){
		    switch (*buf1) {
			case 0x09: {
			    if(users->flags & OPT_EXPTAB){
				tab_buf[tabptr++] = 0x20;
				while (((tabptr) & 0x7) != 0) tab_buf[tabptr++] = 0x20;
				buf1++;
			    }
			    else {
				tab_buf[tabptr++] = *buf1++;
			    }
			    break;
			}
			case 0x0a: {
			    if(users->flags & OPT_BINARY){
				tab_buf[tabptr++] = *buf1++;
			    }
			    else{
				tab_buf[tabptr++] = *buf1++;
				tab_buf[tabptr++] = 0x0d;
				if(users->tcp_socket == SOCK_RTELIAC){
				    tab_buf[tabptr++] = 0x00;
				}
			    }
			    break;
			}
			case 0x0d: {
			    tab_buf[tabptr++] = *buf1++;
			    if(users->tcp_socket == SOCK_RTELIAC){
				tab_buf[tabptr++] = 0x00;
			    }
			    break;
			}
			case 0xff: {
			    tab_buf[tabptr++] = *buf1++;
			    if(users->tcp_socket == SOCK_RTELIAC){
				tab_buf[tabptr++] = 0xff;
			    }
			    break;
			}
			default: {
			    tab_buf[tabptr++] = *buf1++;
			    break;
			}
		    }
		}
		buf1 = tab_buf;
		rval = tabptr;
	    }
	    
	    /* To prevent the network write from hanging indefinitely, the
	       network socket has been set to O_NDELAY which means that if
	       the write would block, the status EWOULDBLOCK or EAGAIN is
	       returned in ERRNO and the write falls through.  If the write
	       does fall through, the sleep statement below should allow the
	       printer time to start flushing the network pipe. */
	       
	    if ((res = write(users->tcp_fd, buf1, rval)) >= 0) {
		if (res == rval){
		    readnew = TRUE;  /* can read new data */
		}
		else{
		    readnew = FALSE;
		    buf1 += res;
		    rval -= res;
		}
	    }
	    else {  /* the write did not succeed at all */
		switch(errno) {
		    case EINTR:{
			errs(users, errno, "process_file:wrtinterr");
			break;
		    }

		    case EWOULDBLOCK:{
			readnew = FALSE;
			sleep(1);
			continue;
		    }
		    
		    default:{
			errs(users, errno, "process_file:wrtfailed");
			rval = PROC_ERR;
			break;
		    }
		}
	    }
	}
    }
    
    if(users->flags & OPT_POST){
	write(users->tcp_fd, "\004", 1);
    }
    else{
	if (!(users->flags & OPT_BINARY)){
	    write(users->tcp_fd, "\f", 1);
	}
    }
    return OK;
}
