/***************************************************************\
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: process_pty.c                                  *
*       Created: Thurs 9 May 10:17:51 1991                     *
*       Abstract:                                              *
*         This module handles the actual communication between *
*         the pseudo-tty interface and the terminal server.    *
*                                                              *
*         This routine waits for input from the pseudo-tty     *
*         master device.  When input arrives, it establishes   *
*         a network connection to the server, (note that we    *
*         could be queued, or the network connection might     *
*         fail) and starts  moving data.  When the user closes *
*         the slave side of the pseudo-tty, we typically get   *
*         some type of I/O error.  We then close the network   *
*         socket, close the master side of the pseudo-tty to   *
*         clear the EOF condition and once again start waiting *
*         for data to arrive.                                  *
*                                                              *
*         Note that it is possible that when we close and      *
*         re-open the pseudo-tty there is a chance that        *
*         someone else will sneak in and allocate our          *
*         pseudo-tty.                                          *
*                                                              *
*       Input:                                                 *
*         As specified per routine.                            *
*                                                              *
*       Output:                                                *
*         User data will be moved between the host system and  *
*         a port on the terminal server.  If errors occur,     *
*         they will be logged in the /usr/spool/rtel status    *
*         file.  (job_status)                                  *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  06-jul-92  Added debugging info.

   RKL  12-may-92  Major badness.  Characters read from the
                   net were not getting back into the pseudo-
		   tty device due to an incomplete if block.

   RKL  22-jan-92  Reorganized and moved miscellaneous support
                   routines to their own files.

   KCS  29-oct-91  First cut.

*/

#include <udpinc.h>

/* Create a couple of global variables used by the signaling
   routines. */

int child_pid, connected;



int process_pty(users)
udp_data *users;
{
    extern hangup(), kid_death();
    
    /* If make_kill_file returns a negative value, either someone else
       is already using the pty pair we selected or a rteld process was
       incorrectly terminated.  Not a good thing. */
    
    if (make_kill_file(users, CREATE_KILL) < 0){
	return(PROC_ERR);
    }
    
    while (TRUE) {
	child_pid = 0;
	signal(SIGCLD, kid_death);
	signal(SIGHUP, hangup);
	
	/* This sleep is here to allow the entire path to reset itself
	   between jobs.  This gives the pseudo-ttys time to reset to
	   a known state and gives the server several seconds between
	   jobs to clean up ports and reset all the queue stuff. */
	
	dbgout("Starting pty job");
	process_data(users);
	sleep(3);
	
	/* Check for errors, and post them so the user can fix problems
	   as they occur per connection. */
	
	if (users->next > 0){
	    wrerr(users, RTEL);
#ifdef DEBUG
            wrerr(users, CONSOLE);
#endif
	    users->next = 0;
	}
    }
}




int process_data(users)
udp_data *users;
{
    int master;
    int rcnt, rval, wcnt; 
    char inbuf[BUFFSIZE], tmpbuf[80], *bufptr;
    
    dbgout("Processing pty data");
    if ((rval = open_pty(users, &master)) >= 0) {
	dbgout("Back from open_pty, success");

	/* If the invoking user did not specify the immediate flag,
	   sit here until data arrives into the pseudo-tty and then
	   start things up.  If the immediate flag was specified, open
	   the connection immediately since the host might just want
	   to start reading from the network device.  */

	if(users->pty_flags & CONN_IMED){
	    rcnt = 0;
	}
	else{
	    while(TRUE){
		dbgout("Just before read");
		rcnt = read(master, inbuf, BUFFSIZE);
#ifdef DEBUG
		sprintf(dbgbuf, "Read returned %d, errno %d", rcnt, errno);
		dbgout(dbgbuf);
#endif
		if(rcnt > 0)break;
		if((errno != EIO) && (errno =! EINTR)){
		    errs(users, errno, "process_pty:badmstrd");
		    return(PROC_ERR);
		}
	    }
	}
	
	/* We are here to open up a connection to the server.  Go ahead
	   and try to talk over the net.  */
	
	users->flags = 0;
	if(users->pty_flags & QUEUE_PTY){
	    users->flags = ALLOW_QUE;
	    dbgout("Allowing queueing");
	}
	
	while(TRUE){
	    dbgout("Attempting to connect to service");
	    rval = connect_service(users, MAKE_CONN);
	    if(rval < OK){
		errs(users, RTEL_NOCONN, "process_pty:badcon");
		goto DONE;
	    }
	    if(rval == OK){
		connected = TRUE;
		break;
	    }
	    waiting(users);
	}
	
	/* Time to write the first first data buffer to the net. Note that if this
	   is an "immediate" connection, rcnt will be zero and everything will work. */

	bufptr = inbuf;
	while(rcnt != 0){
#ifdef DEBUG
	    sprintf(dbgbuf, "Writing %d bytes from %x", rcnt, bufptr);
	    dbgout(dbgbuf);
#endif
	    if((rval = write(users->tcp_fd, bufptr, rcnt)) < 0) { 
		errs(users, errno, "process_pty:initwrite");
		goto DONE;
	    }
	    rcnt -= rval;
	    bufptr += rval;
	    if(rcnt > 0){
		sleep(1);
	    }
	}
	
	/* Now fork off into two processes, one reading from the server and
	   one writing to the server.  Note that this really should be able
	   to be done with a single select reading and writing from the network
	   fd, but several machines seem to have problems with exception
	   condition handling on pseudo-ttys.  This is safer.  */
	
	if ((child_pid = fork()) == 0){
	    movedata_net_to_host(users, master);
	    child_pid = 0;
	    exit(0);
	}
	else if (child_pid > 0) {
	    movedata_host_to_net(users, master);
	}
	else { /* fork failed... */
	    rval = -1;
	}
    }

    DONE:	
    if(child_pid > 0){
	kill(child_pid, SIGKILL);
	child_pid = 0;
    }
	
    close(master);
    close(users->tcp_fd);
    close(users->udp_fd);
    return(rval);
}





movedata_host_to_net(users, master)
udp_data *users;
int master;
{
    char *bufptr, inbuf[BUFFSIZE];
    int wcnt, rcnt;
    
    while (connected){
	if ((rcnt = read(master, inbuf, BUFFSIZE)) <= 0) {
	    if (errno == EINTR) { /* got interrupt, no need to die */
		continue;
	    }
	    if ((errno != ENXIO) && (errno != EIO)){
		errs(users, errno, "process_pty:netread");
	    }
	    return(PROC_ERR);
	}
	
	bufptr = inbuf;
	while(rcnt != 0){
	    if((wcnt = write(users->tcp_fd, bufptr, rcnt)) < 0) { 
		errs(users, errno, "process_pty:netwrite");
		return(PROC_ERR);
	    }
	    rcnt -= wcnt;
	    bufptr += wcnt;
	    if(rcnt > 0){
		sleep(1);
	    }
	}
    }
}






movedata_net_to_host(users, master)
udp_data *users;
int master;
{
    char *bufptr, inbuf[BUFFSIZE];
    int wcnt, rcnt;
    
    while(connected){
	if ((rcnt = read(users->tcp_fd, inbuf, BUFFSIZE)) <= 0){
	    if (errno == EINTR){
		continue;
	    }
	    errs(users, errno, "processs_pty:nthrderr");
	    return(PROC_ERR);
	}

	bufptr = inbuf;
	while(rcnt > 0){
	    if((wcnt = write(master, bufptr, rcnt)) < 0){ 
		errs(users, errno, "process_pty:nthwrterr");
		return(PROC_ERR);
	    }
	    rcnt -= wcnt;
	    bufptr += wcnt;
	    if(rcnt > 0){
		sleep(1);
	    }
	}
    }
}
