/***************************************************************\
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: rteld                                          *
*       Created: Thurs 9 May 10 17:51 1991                     *
*                                                              *
*       Abstract:                                              *
*         This is the user interface to the wonderful world    *
*         of rtel's pseudo-tty communications devices.         *
*                                                              *
*         There are six major command catagories this program  *
*         handles:                                             *
*                                                              *
*           TALK --- This option moves data between a host     *
*                    pseudo-tty device and a port on a server. *
*                                                              *
*           SETUP -- This option forks off a daemon process    *
*                    for each line in the server_hosts file    *
*                    that starts with a "/".  This would       *
*                    typically be used in a system startup     *
*                    file.                                     *
*                                                              *
*           KILL --- This option deletes currently running     *
*                    rteld processes and removes their         *
*                    associated kill files.                    *
*                                                              *
*           REVIEW - This option shows what rteld process are  *
*                    currently running and lists the pseudo-   *
*                    pairs they have allocated.                *
*                                                              *
*           CLEAN -- This option removes kill files associated *
*                    with stale rtel daemons.  Stale daemons   *
*                    occur when users do stupid things like    *
*                    kill -9 on rteld processes.               *
*                                                              *
*           VERS --- This option will simply display the rtel  *
*                    version level.                            *
*                                                              *
*       Input:                                                 *
*         The user passes necessary parameters through the     *
*         startup command line.                                *
*                                                              *
*       Output:                                                *
*         If no errors occur, this routine will generate no    *
*         output.  If a processing error occurs, appropriate   *
*         error messages will be logged to the /usr/spool/rtel *
*         job_status file and also to the invoking terminal    *
*         if applicable.                                       *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  13-jul-92 Added doing automatic cleanup when doing a
                  system startup command.

   RKL  06-jun-92 Added debugging info.

   RKL  29-jun-92 Added -v option to show rtel version level.

   RKL  11-apr-92 Added -i (immediate) option.

   KCS  29-oct-91 added new option -t, which causes telnet
                  IAC padding and interpretation to work.

   KCS  10-sep-91 The parser complained because the users
                  structure was not initialized. 
			    
*/

#include <udpinc.h>

main(argc,argv)
int argc;
unsigned char* argv[];
{
    udp_data users;
    unsigned short euid;
    int process;

    /* Zero the users structure and parse the users input. */

    bzero(&users, sizeof (udp_data));
    users.tcp_socket = SOCK_RTEL;

    process = rteld_parse(argc, argv, &users);

    switch (process){
	case TALK:{
	    dbgout("Doing talk ");
	    do_talk(&users);
	    break;
	}

	case SETUP:{
	    if (users.host[0] || users.svcnm[0] || users.ttydevm[0]){
		printf("Invalid option specified along with setup\r\n");
	    }
	    else{
		if ((euid = geteuid()) != 0) {
		    printf("You must have root privs for this option\r\n");
		}
		else{
		    dbgout("Doing cleanup and then setup");
		    do_cleanup(&users);
		    do_setup(&users);
		}
	    }
	    break;
	}
	
	case CLEAN:{
	    dbgout("Doing cleanup");
	    do_cleanup(&users);
	    break;
	}
	
	case KILL:{
	    if (users.host[0] || users.svcnm[0] || !users.ttydevm[0]){
		printf("Invalid option specified along with kill\r\n");
	    }
	    else{
		dbgout("Doing kill");
		do_kill(&users);
	    }
	    break;
	}

	case REVIEW:{
	    if (users.host[0] || users.svcnm[0] || users.ttydevm[0]){
		printf("Invalid option specified along with kill\r\n");
	    }
	    else{
		dbgout("Doing review");
		do_review(&users);
	    }
	    break;
	}

	case VERS:{
	    printf("\r\n\nRTEL Version %s\r\n\n", VERSION);
	    break;
	}

	default:{
	    printf("Typical command line formats\r\n");
	    printf("----------------------------\r\n");
	    printf("rteld -cservice@host [options]\r\n");
	    printf("rteld -d/dev/ttyXX [options]\r\n");
	    printf("rteld -c:pnum@host -d/dev/ttyXX [options]\r\n");
	    printf("rteld -kall\r\n");
	    printf("rteld -k/dev/ttyxx\r\n");
	    printf("\r\n");
	    printf("Commands and Options\r\n");
	    printf("--------------------\r\n");
	    printf("-b      spawn rteld as a system daemon\r\n");
	    printf("-c      specify service/server or port number/server\r\n");
	    printf("-d      use specified psuedo-tty device\r\n");
	    printf("-f      remove stale rteld kill files\r\n");
	    printf("-h      display this help message\r\n");
	    printf("-i      immediately form connection\r\n");
	    printf("-k      kill rteld for specified device (or all devices)\r\n");
	    printf("-p      specify password to use with connection\r\n");
	    printf("-q      allow queueing to busy services\r\n");
	    printf("-r      display currently running rtel daemons\r\n");
	    printf("-s      start RTEL daemons as specified in server_hosts\r\n");
	    printf("-t      interpret telnet IAC codes\r\n");
	}
    }
    
    /* Check for errors. */
    if (users.next > 0){ 
	wrerr(&users, RTEL);
    }
    exit (0);
}
