/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: signals                                        *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         These routines handle setting up and collecting      *
*         signals in both the SYSV and BSD environments.       *
*                                                              *
*       Input:                                                 *
*         -- none --                                           *
*                                                              *
*       Output:                                                *
*         -- none --                                           *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  17-jun-92  Upgraded waiting routine to actually wait
                   for data coming back from the server on the
		   udp channel.
   
   RKL  07-jan-92  Cleanup and completely remove the itermval
                   stuff.

   RKL  03-jul-91  Conditionalized the use of the itimerval
                   routines since SCO does not seem to have
		   them.

   RKL  17-may-91  Added signal routines hangup and kid_death
                   for support of RTEL II.

   RKL  10-jan-91  Added routine pause_abit which gives network
                   buffered data a chance to clean out a bit.

   RKL  09-jan-91  Changed RTAP to RTEL.

*/

#include <udpinc.h>



/*  Collect defunct children when a parent process
    terminates.  This is used when the daemon wants to
    become a daemon process. */

int pickup()
{
    int status, rval;
#ifdef SIGTSP
    rval = wait3 (&status, WNOHANG, (struct rusage*) 0);
#else
    rval = wait (&status);
#endif
}



int getchild()
{
    pickup();
#ifdef SIGCHLD
    signal (SIGCHLD, getchild);
#else
#ifdef SIGCLD
    signal (SIGCLD, getchild);
#endif
#endif
}



/* This routine is used by the queueing routines to keep trying
   a connection every once in a while.  This allows for recovery
   from a server crashing and not having a queued job when it
   comes back up.  */

int waiting(users)
udp_data *users;
{
    struct timeval KevClock;
    fd_set efd, rfd;
    char buff[128];
    int stat;
    
    /* This select will fall through if this process gets information
       back from the server via the UDP channel.  In addition, a timer
       is set to expire every 60 seconds so that we will force a retry
       to the printer service.  This will catch the case of the server
       rebooting and losing track of all information in the queue.

       The UDP message from the server will be of the form:
	   
            1 byte  -- service name length
            n bytes -- service name
	    1 byte  -- port number low
	    1 byte  -- port number high

       This will be check to confirm that it matches the information
       we originally supplied in our request. */
    
    FD_ZERO(&efd);
    FD_ZERO(&rfd);

    FD_SET(users->udp_fd, &efd);
    FD_SET(users->udp_fd, &rfd);

    KevClock.tv_sec = 60;
    KevClock.tv_usec = 0;

    if((stat = select(getdtablesize(), &rfd, 0, &efd, &KevClock)) < OK){
	errs(users, errno, "waiting:badselect");
	return(PROC_ERR);
    }
    
    if(FD_ISSET(users->udp_fd, &efd)){
	errs(users, RTEL_SELEXCEPT, "waiting:selexcept");
	return(PROC_ERR);
    }

    if(FD_ISSET(users->udp_fd, &rfd)){
	if (read(users->udp_fd, buff, sizeof(buff)) < OK){
	    errs(users, errno, "waiting:netread");
	    return(PROC_ERR);
	}
	else {
	    dbgout("Got UDP queue message");
	    return(OK);
	}
    }
}



/* This routine is used by the connection routine to wait for a signal
   from the UDP daemon.  When a signal comes in, all this guy does is
   return.  */

int sigcont()
{
    signal(SIGUSR1, SIG_IGN);
}



/* This routine is called by the connect_service routine if a connection
   to a server does not succeed within a certain time period.  This will
   occur if a connection is attempted to a dead terminal server.  */

int connect_timeout()
{
    extern char FILENAME[MAXLEN], SERVER[MAXLEN];
    char output[132];

    sprintf(output, "%%TSRVDWN: Server %s not responding to connection request", SERVER);
    dbgout(output);

    timestamp(jobstat, FILENAME);
    err_handler(jobstat, output, "signals:connect_timeout");
    dbgout("Exiting due to inability to connect to server");
    exit(-1);
}



/* This routine is used by the various xx_filter programs to ensure that
   any extraneous signals are ignored.  */

int signal_stuff ()
{
    
#ifdef SIGALRM
    signal (SIGALRM, SIG_IGN);
#endif
    
#ifdef SIGUSR1
    signal (SIGUSR1, SIG_IGN);
#endif
    
#ifdef SIGUSR2
    signal (SIGUSR2, SIG_IGN);
#endif
} 



/***************************************************************
  
  For RTEL II, there are two types of signals to worry about.
  
  The first is in the case the kid gets killed off.  if this
  happens, then there would be no connection on the network read
  side.  Thus, the CONNECT flag gets set to false (provided a
  signal is received from the dying child) and the next write
  that is attempted from the parent (the master pty write to
  net) will realise that its child is dead and exit.
  
  The second case is the case of killing off the parent.
  Hopefully the user will use the supplied file and the rtel
  -k(ill) feature.  However, if a kill -9 is used, the child
  will stay up and running, and the kill file will remain in
  existence.  In the case of a kill -1, or rtel -k(ill), or
  execution of the kill file, a sighup signal will result,
  causing a signal capture, a kill -9 of the child, and
  unlinking (deleting) the kill file.
  
  ***************************************************************/

int kid_death()
{
    extern char connected;
    pickup();
    connected = FALSE;
#ifdef SIGCHLD
    signal (SIGCHLD, kid_death);
#else
#ifdef SIGCLD
    signal (SIGCLD, kid_death);
#endif
#endif
}


int hangup()
{
    extern child_pid;
    if (child_pid > 0){
	kill (child_pid, SIGKILL);
    }
    exit (0);
}
