/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Rich Lyman                                     *
*       Module: talk                                           *
*       Created: Wed Aug 15 12:05:10 1990                      *
*       Abstract:                                              *
*         This program demonstrates how to use the object      *
*         modules supplied by Gordian to create a simple       *
*         Reverse Telnet Application (RTEL)                    *
*                                                              *
*         The example program will simply allow a user on a    *
*         UNIX system to connect to a specific port on an ETS  *
*         server and dump out some data and then read some     *
*         user input and display it on the screen.             *
*                                                              *
*       Input:                                                 *
*         The user is prompted for the majority of the input   *
*         needed for this routine.  However, the target server *
*         must be in the /etc/hosts file for the connection    *
*         to succeed.                                          *
*                                                              *
*         A user may also specify the required input as part   *
*         of the command line.  The needed parameters are:     *
*                                                              *
*            argv[1] -- server name                            *
*            argv[2] -- service name                           *
*            argv[3] -- service password (optional)            *
*                                                              *
*         Note that the only way to specify a password is to   *
*         attempt the connection using the command line.       *
*                                                              *
*       Output:                                                *
*         A session will be established with the specified     *
*         server/service combination.  It will spew a small    *
*         amount of data to the server and then read from the  *
*         server port until the user types an uppercase Q or   *
*         program is killed.  No file I/O will occur.          *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  17-jun-92  Adjusted for potential TLI support.

   KCS  07-may-91  Added message field to errs() call, and err_handler.

   The file udpinc.h contains all the RTEL specific definitions required.
   In addition, it contains structures that may not be defined on all
   systems.  (These structures are conditionally included based on -D
   directives on the command line.) */

#include <udpinc.h>
#include <stdio.h>

main(argc,argv)
int argc;
unsigned char* argv[];
{
  fd_set   rd,except;
  udp_data ud;
  char inbuf[80], 
  outbuf[80],
  errbuf[128];
  int stat;
  
  /* Initialize the error array pointer.  Errors for each connection
     attempt will be batched and printed when the program exits.  */
  
  ud.next=0;
  
  /* Parse the command line to see if the user specified the server,
     service name and password fields.  If not, prompt the user. */
  
  if(argc > 1)
    strcpy(ud.host, argv[1]);
  else{
    printf("Please enter server name: ");
    scanf("%s16", ud.host);
  }
  
  if(argc > 2)
    strcpy(ud.svcnm, argv[2]);
  else{
    printf("Please enter service name: ");
    scanf("%s16", ud.svcnm);
  }
  
  if(argc > 3)
    strcpy(ud.passwd, argv[3]);
  else
    ud.passwd[0] = 0;
  
  printf("Connecting to server %s, service %s, using password %s\n",
	 ud.host, ud.svcnm, ud.passwd);
  
  /* Now actually request a connection to the service.  In addition to
     the server, service and password, the flags field must be set up.
     Setting the flags to zero means no queueing and this is the first
     connection attempt. */
  
  ud.flags = 0;
  if ((stat=connect_service(&ud, MAKE_CONN)) == OK) {
    
    /* The connection has been made and the network socket has been
       returned in ud.sock.  This value can be used in all subsequent
       network I/O. */
    
    strcpy(outbuf, "\r\n\nWelcome to the exciting world of RTEL...");
    
    if((stat = write(ud.tcp_fd, outbuf, strlen(outbuf))) < 0){
      errs(&ud, errno, "talk:main, bad write to socket");
      fprintf(stderr,"sock write err: %d, \r\n",stat);
      goto OUT;
    }
    
    /* Now read characters from the net and echo them to the display
       until either a capital "Q" is entered or the program is killed. */
    
    while (TRUE) {
      FD_ZERO(&rd);
      FD_ZERO(&except);
      FD_SET(ud.tcp_fd, &rd);
      FD_SET(ud.tcp_fd, &except);
      
      stat = select (getdtablesize(), &rd, 0, &except, 0);
      
      if (stat < OK) {
	errs(ud, errno, "talk:main, bad select");
	break;
      }
      
      if (FD_ISSET(ud.tcp_fd, &rd)) {
	if ((stat = read(ud.tcp_fd, inbuf, 80)) <= 0) {
	  fprintf(stderr,"Just after bad read, %d", stat);
	  if (errno == EINTR) continue;   /* ignorable interrupt */
	  errs(ud,RTEL_FINNED, "conn_srvc:connect_service, connection completed");
	  sprintf(errbuf, "conn_srvc:  connect_service, err mssg if connect abort");
	  errs(ud, errno, errbuf);
	  break;
	}
	while (stat--){
	  fprintf(stderr,"%c", inbuf[stat]);
	  if(inbuf[stat] == 'Q') goto OUT;
	}
      }
    }
  }
  
  /* Close the network socket.  Everything is then cleaned up
     and the server is ready to accept another connection. */
  
 OUT:
  if((stat = close(ud.tcp_fd)) < 0){
    errs(&ud, errno,"talk:  main, error closing socket");
  }
  
  /* Check for errors. */
  if(ud.next > 0){
    printf("\nThe following errors have occured\n");
    printf("------------------------------------------------------------\n");
    wrerr(&ud, TERMINAL);
  }
  printf("\r\n");
}
